/*  commandir_record  Version 0.5
    Initial Release Feb 9 2011
    0.2 - March 2011 - Added -g switch to specify gap
    0.3 - June 16th, 2011 - Added -s switch to specify a specific CommandIR
    0.4 - July 28th, 2011 - Added -p switch for named pipe output to /tmp/commandir0
    0.5 -  - Removed -p switch; moved to new commandird daemon
 */

#include "cmdirutils_version.h"
#include "libcmdir.h"
#include "libtcp_client.h"
#include "libcmdir_record.h"

extern char * rec_filename;
extern int gap_size;
extern unsigned char displayOnly;
extern unsigned char pipeOnly;
// extern unsigned char incomingData[16384];

unsigned char standalone = 1;

void waitForTcpRecordData(void);

extern char determining_gap;


void waitForTcpRecordData() 
{
  unsigned char dat[16384];
  int displayed;
  int len;
  int c;
  
  len = networkClientRecv(dat, 16384);
  if(displayOnly == 1)
  {
    printf("IR codes from network CommandIR (press <enter> or <Ctrl+C> to quit):\n");
  } else {
    printf("Press <enter> to accept code\n");
  }
  
  while(1)
  {
    c = kbhit();
    if(c) {
      c = getchar();
      while(kbhit())  // clear the keyboard buffer, incase they hit keys other than enter
        c = getchar();
      break;
    }
    // len should be returned when we get a terminating \0 so we know we have a full HexString
    // Note that the network might return multiple signals!
    if(len > 0) {
      dat[len] = '\0';  // Zero terminate it at the correct place!
      displayed = 0;
      while(displayed < len) {
        // gap is encoded in file!
        displayAndPipeSignal(&dat[displayed], strlen(&dat[displayed]), 0);  
        displayed += strlen(&dat[displayed]) + 1;  // +1?
      }
    }
#ifdef WIN
    Sleep(5);
#else
    usleep(5000);
#endif
    len = networkClientRecv(dat, 16384);
//    printf("Received %d bytes: (%p) %s\n", len, dat, dat);
  }
  // When it's done, return and close
}

int main(int argc, char **argv)
{
  int x, y, c, gap_specified = 0;
  struct commandir_device * pcd;
  char cmdLine[16384];
  
  if(argc == 1)
  {
    printf("commandir_record %s\n\n", utils_version);
    printf(" Record IR codes from a CommandIR III\n\n");

    printf("Usage: commandir_record [-n] [-sCOMMANDIR_NAME] [-gGAP_SIZE] BUTTON_NAME [BUTTON_NAME] ...\n");
    printf("       commandir_record [-n] [-sCOMMANDIR_NAME] [-gGAP_SIZE] -d\n\n");
    printf(" -gGAP_SIZE - in microseconds (us): calibrate the recorder to distinguish between repeat\n");
    printf("              signals. Any spaces larger than -g are consider gap.  Default: 50000\n");
//    printf("            - If -g is not specified, commandir_record will try to determine it.\n");
    printf(" -d - Display Hex String only, do not save to file.\n");
    printf(" -o - Only return gap values, in Hex and microseconds.\n");
    printf(" -f - Do not add detected gap values to Hex codes.\n");

    printf(" -sCOMMANDIR_NAME - Record from a specific CommandIR (Firmware >=3.42 required)\n");
    printf(" -n[Network_Host_Name] - Record from a CommandIR via the 'commandird' daemon.\n");
    printf("      Defaults to 'localhost' if no other is specified. \n");
    return 0;
  }
  
  // Is this a network command, or a stand-alone?
  // commandird only cares about -s and -g
  // Only for us: -d -p -n
  for(x=1;x<argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'n':
          // Network Command
              // Network send - default to localhost if none specified
          if(pipeOnly == 1) 
          {
            printf("Cannot operate in both network and pipe mode\n");
            return -1;
          }
          networkSend = 1;
          if(strlen(argv[x]) > 2) {
            strncpy(networkSendName, &argv[x][2], strlen(argv[x]));
            networkSendName[strlen(argv[x])] = '\0';
          } else {
            strncpy(networkSendName, localhostPtr, strlen(localhostPtr));
          }

          /* CONNECT */
          if(!networkClientConnect(networkSendName)) {
            printf("Unable to connect to %s\n", networkSendName);
            return -1;
          }
          
          /* Make this easy: Pass the whole command line to the commandird server */
          c = strlen("commandir_record ");
          strncpy(cmdLine, "commandir_record ", c);
          for(y=1;y<argc;y++) {
            if(argv[y][0]=='-' && argv[y][1]=='n')
              continue; // do not send the Network send flag
            if(argv[y][0]=='-' && argv[y][1]=='d')
              continue; // do not send the display flag
            if(argv[y][0]=='-' && argv[y][1]=='s') {
              printf("Cannot specify a specific CommandIR in network mode (yet)\n");
              continue; 
            }
            if(argv[y][0]=='-' && argv[y][1]=='g')
              gap_specified = 1;
              
            if(argv[y][0]=='-') // No file names to commandird
              c+= sprintf(&cmdLine[c], "%s ", argv[y]);
          }
          
          if(gap_specified == 0) {
            // Make sure the gap is reset to the default:
            printf("Setting gap to default: %d microseconds\n", gap_size);
            c+= sprintf(&cmdLine[c], "-g%d", gap_size);
          }
          
          cmdLine[c++] = '\0';
          networkClientSend(cmdLine, c);
          
          break;
        case 'd':
          // Display Only - don't save to a file
          displayOnly = 1;
          break;
/*        case 'p':
          // Send to pipe - don't save to a file
          if(networkSend == 1) 
          {
            printf("Cannot operate in both network and pipe mode\n");
            return -1;
          }
          pipeOnly = 1;
          break;
 */
      }
    } else {
      // If it's not a switch, it's a filename
      if(displayOnly) {
        printf("Do not specify a filename if you want to display codes with -d.\n");
        return -1;
      }
      if(pipeOnly) {
        printf("Do not specify a filename if you want to pipe with -p.\n");
        return -1; 
      }
      rec_filename = argv[x];
      printf("Recording %s ...\n", rec_filename);
      if(networkSend == 1) 
      {
        // Multiple recording from via network
        
        waitForTcpRecordData();        
      } /* else {   - let execute_commandir_record_arg handle this
        // Stand-alone, easy recording
        if(pcd)
          commandir_rec(pcd->cmdir_udev);
        else
          commandir_rec(NULL);
      } */  
    }
  }
  
  if(networkSend == 1) 
  {
    if(displayOnly == 1) 
    {
      /* This case is for continuous operation - display or pipe only */
      waitForTcpRecordData();
    }
    networkClientClose();
    return 0;  
  } else {
    printf("commandir_record %s\n\n", utils_version);
  
  }
  
//  printf("commandir_record shoud not run execute_commandir_record_arg\n");
//  return 0;
  
  // Stand-alone local command:
  execute_commandir_record_arg(argc, argv);
    
  return 0;
  
}




